from django.utils.translation import ugettext_lazy as _
from django.db.models import Model

from .exceptions import PermAppException
from .permissions import permissions_manager


class ModelPermissionBackend(object):

    supports_object_permissions = True
    supports_anonymous_user = True
    supports_inactive_user = True

    def authenticate(self, username, password):
        """
        This backend does not authenticate
        """
        return None

    def has_perm(self, user_obj, perm, obj=None):

        # Non-existing and inactive users never get permission
        if not user_obj or not user_obj.is_active:
            return False

        # If obj is an instance, get the model class
        if not obj:
            obj = None
            model = None
        elif isinstance(obj, Model):
            model = obj.__class__
        elif issubclass(obj, Model):
            model = obj
            obj = None
        else:
            raise PermAppException(_('Unexpected value for model.'))

        # If permission is in dot notation,
        perm_parts = perm.split('.')
        if len(perm_parts) > 1:
            # Keep only the last part of the permission (without application name)
            perm_app, perm = perm_parts
            # Make sure permission and object application are the same
            if model:
                model_app = model._meta.app_label
                if perm_app != model_app:
                    raise PermAppException(
                        _("App mismatch, perm has '%(perm_app)s' and model has '%(model_app)s'" % {
                            'perm_app': perm_app,
                            'model_app': model_app,
                        })
                    )

        # Get the ModelPermissions object
        object_permissions = permissions_manager.get_permissions(model, user_obj, perm, obj)

        # No ModelPermissions means no permission
        if not object_permissions:
            return False

        # Check the permissions
        return object_permissions.has_perm()



from .permissions import permissions_manager

# Decorator for models
def permissions(permissions_class):
    def wrap(model):
        permissions_manager.register(model, permissions_class)
        return model
    return wrap


# Decorator for permissions class
def permissions_for(model):
    def wrap(permissions_class):
        permissions_manager.register(model, permissions_class)
        return permissions_class
    return wrap



from __future__ import unicode_literals

import re

from django.db.models import Model
from django.template import Library, Node, TemplateSyntaxError, Variable, VariableDoesNotExist
from django.utils.translation import ugettext_lazy

from ..utils import get_model


QUOTED_STRING = re.compile(r'^["\'](?P<noquotes>.+)["\']$')

register = Library()


def handle_var(value, context):
    stringval = QUOTED_STRING.search(value)
    if stringval:
        return stringval.group('noquotes')
    else:
        try:
            return Variable(value).resolve(context)
        except VariableDoesNotExist:
            return value


@register.tag
def ifperm(parser, token):
    # Separate tag name and parameters
    try:
        parts = token.split_contents()
        tag = parts[0]
        perm = parts[1]
        try:
            obj_or_model = parts[2]
        except IndexError:
            obj_or_model = None
    except (ValueError, TypeError):
        raise TemplateSyntaxError(ugettext_lazy("'%(tag)s' tag takes two parameters") % {'tag': tag})

    default_states = ['ifperm', 'else']
    end_tag = 'endifperm'

    # Place to store the states and their values
    states = {}

    # Let's iterate over our context and find our tokens
    while token.contents != end_tag:
        current = token.contents
        states[current.split()[0]] = parser.parse(default_states + [end_tag])
        token = parser.next_token()

    return PermNode(states, perm, obj_or_model)


class PermNode(Node):
    def __init__(self, states, perm, obj_or_model):
        self.states = states
        self.perm = perm
        self.obj_or_model = obj_or_model

    def render(self, context):
        perm = handle_var(self.perm, context)
        model = handle_var(self.obj_or_model, context)

        if not isinstance(model, Model):
            model = get_model(model, raise_exception=True)

        if context['request'].user.has_perm(perm, model):
            index = 'ifperm'
        else:
            index = 'else'

        try:
            return self.states[index].render(context)
        except KeyError:
            return ''



from __future__ import unicode_literals

from django.db.models import Model, get_model as django_get_model
from django.utils.translation import ugettext as _

from .exceptions import PermAppException


def get_model(model, raise_exception=False):
    if isinstance(model, basestring):
        # If model is a string, find the appropriate model class
        try:
            app_name, model_name = model.split('.')
        except ValueError:
            model_class = None
        else:
            model_class = django_get_model(app_name, model_name)
    else:
        # Assume we have been given a model class or instance
        model_class = model

    # Test is this is an instance or subclass of Model
    try:
        is_model = issubclass(model_class, Model)
    except TypeError:
        is_model = False

    # Handle failure
    if not is_model:
        if raise_exception:
            raise PermAppException(_('%(model)s is not a Django Model class.' % {
                'model': model
            }))
        return None

    # Return the result
    return model_class



from django.core.exceptions import PermissionDenied

from .permissions import load_permissions
from .http import HttpException, HttpForbidden


class ModelPermissionsMiddleware(object):

    def __init__(self, *args, **kwargs):
        super(ModelPermissionsMiddleware, self).__init__(*args, **kwargs)
        # Run load_permissions exactly once
        load_permissions()

    def process_exception(self, request, exception):
        # Convert standard PermissionDenied to our own HttpForbidden
        if isinstance(exception, PermissionDenied):
            exception = HttpForbidden(unicode(exception))
        # Make our own HttpException return its response
        if isinstance(exception, HttpException):
            response = exception.render_to_reponse(request)
            if response:
                return response
        # Nothing more to do here
        return None


from __future__ import unicode_literals

from .permissions import permissions_manager


def get_perm_queryset(model, user, perm):
    permissions = permissions_manager.get_permissions(model, user, perm, raise_exception=True)
    return permissions.get_queryset()



from django.core.handlers.wsgi import STATUS_CODE_TEXT
from django.shortcuts import render

from .exceptions import PermException


class HttpException(PermException):
    message = ''
    status_code = 500 # because this means you've instantiated the wrong thing
    headers = {} # extra headers for the final HTTP response

    def __init__(self, *args, **kwargs):
        super(HttpException, self).__init__(*args, **kwargs)
        self.message = unicode(self)

    def get_templates(self):
        # Get template
        return ["perm/http_%s.html" % self.status_code, "perm/http.html"]

    def get_context(self):
        return {
            'message': self.message,
            'http_status_code': self.status_code,
            'http_status_text': STATUS_CODE_TEXT.get(self.status_code, 'UNKNOWN'),
            'http_exception': self,
        }

    def render_to_reponse(self, request):
        # Basic response
        response = render(request, self.get_templates(), self.get_context())
        # Set status code
        response.status_code = self.status_code
        # Add headers
        if self.headers:
            for k, v in self.headers.items():
                response[k] = v
        # Augment response
        self.augment_response(response)
        # Return result
        return response

    def augment_response(self, response):
        # Overwrite this to augment the response
        pass


class HttpBadRequest(HttpException):
    status_code = 400


class HttpForbidden(HttpException):
    status_code = 403



class PermException(Exception):
    """
    Something goes wrong within perm
    """
    pass


class PermAppException(PermException):
    """
    Something goes wrong withtin perm, but it it something that might happen because of they way the app works
    """
    pass


class PermQuerySetNotFound(PermAppException):
    """
    The queryset we were looking for was not defined in the app
    """
    pass



from django.views.generic import DetailView, UpdateView, CreateView, ListView, DeleteView

from .http import HttpForbidden
from .shortcuts import get_perm_queryset


class PermMixin(object):
    perm = None


class PermSingleObjectMixin(PermMixin):
    def dispatch(self, request, *args, **kwargs):
        if isinstance(self, CreateView):
            obj = self.model
            if self.perm and not self.request.user.has_perm(self.perm, obj):
                raise HttpForbidden()
        return super(PermSingleObjectMixin, self).dispatch(request, *args, **kwargs)

    def get_object(self, *args, **kwargs):
        if isinstance(self, CreateView):
            obj = self.model
        else:
            obj = super(PermSingleObjectMixin, self).get_object(*args, **kwargs)
        if self.perm and not self.request.user.has_perm(self.perm, obj):
            raise HttpForbidden()
        return obj


class PermMultipleObjectMixin(PermMixin):
    def get_queryset(self, *args, **kwargs):
        qs = get_perm_queryset(self.model, self.request.user, self.perm)
        try:
            super_qs = super(PermMultipleObjectMixin, self).get_queryset(*args, **kwargs)
        except AttributeError:
            super_qs = None
        else:
            qs = qs.filter(pk__in=super_qs)
        return qs


class PermDetailView(PermSingleObjectMixin, DetailView):
    perm = 'view'


class PermUpdateView(PermSingleObjectMixin, UpdateView):
    perm = 'change'


class PermCreateView(PermSingleObjectMixin, CreateView):
    perm = 'create'


class PermListView(PermMultipleObjectMixin, ListView):
    perm = 'list'


class PermDeleteView(PermSingleObjectMixin, DeleteView):
    perm = 'delete'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import os
import sys

from setuptools import setup


name = 'django-perm'
package = 'perm'
description = 'Class based Django permissions for Django models.'
url = 'https://github.com/dyve/django-perm'
author = 'Dylan Verheul'
author_email = 'dylan@dyve.net'
license = 'Apache License 2.0'
install_requires = []
classifiers = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: Apache Software License",
    "Programming Language :: Python",
    "Operating System :: OS Independent",
    "Topic :: Software Development :: Libraries",
    "Topic :: Utilities",
    "Environment :: Web Environment",
    "Framework :: Django",
]

def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print "You probably want to also tag the version now:"
    print "  git tag -a %(version)s -m 'version %(version)s'" % args
    print "  git push --tags"
    sys.exit()


setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=install_requires,
    classifiers=classifiers,
)



from __future__ import unicode_literals

from django.db.models.query import EmptyQuerySet
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from perm.exceptions import PermAppException, PermQuerySetNotFound
from perm.utils import get_model


class ModelPermissionsManager(object):
    """
    Singleton object to hold ModelPermissions classes for objects
    """

    _registry = {}

    def register(self, model, permissions_class):
        model = get_model(model)
        self._registry[model] = permissions_class
        return model

    def register_permissions_class(self, permissions_class, model):
        self.register(model, permissions_class)
        return permissions_class

    def get_permissions(self, model, user_obj, perm, obj=None, raise_exception=False):
        model = get_model(model)
        permissions_checker_class = self._registry.get(model, None)
        if not permissions_checker_class:
            if raise_exception:
                raise PermAppException(_('No permissions registered for %(model)s.' % {'model': model}))
            return None
        permissions = permissions_checker_class(model, user_obj, perm, obj)
        return permissions


class ModelPermissions(object):
    """
    Class is instantiated once a permission has to be checked.
    The check itself is done by calling the has_perm() method.
    """
    model = None
    user = None
    perm = None
    obj = None

    def __init__(self, model, user_obj, perm, obj=None, *args, **kwargs):
        """
        Set the properties
        """
        super(ModelPermissions, self).__init__(*args, **kwargs)
        self.model = model
        self.user = user_obj
        self.obj = obj
        self.perm = perm

    def get_queryset(self):
        """
        Get method get_queryset_perm_PERM or return None
        """
        try:
            method = getattr(self, 'get_queryset_perm_%s' % self.perm)
        except AttributeError:
            raise PermQuerySetNotFound(_('Permissions for %(model)s do not include queryset for %(perm)s.' % {
                'model': self.model,
                'perm': self.perm
            }))
        return method()

    def _has_perm_using_method(self):
        """
        Test the method has_perm_PERM()
        """
        try:
            method = getattr(self, 'has_perm_%s' % self.perm)
        except AttributeError:
            return False
        return method()

    def _has_perm_using_queryset(self):
        """
        Test to see if obj appears in get_perm_PERM_queryset()
        """
        try:
            return self.get_queryset().filter(pk=self.obj.pk).exists()
        except PermQuerySetNotFound:
            return False

    def has_perm(self):
        """
        Test using direct method and queryset
        """
        return self._has_perm_using_method() or self._has_perm_using_queryset()


def load_permissions():
    """
    This is called once from the ModelPermissionsMiddleware.__init__ to discover all permissions in all applications
    """
    try:
        if load_permissions.done:
            return
    except AttributeError:
        pass
    load_permissions.done = True
    for app in settings.INSTALLED_APPS:
        try:
            __import__('%s.permissions' % app)
        except ImportError:
            pass


# Instantiate the singleton
permissions_manager = ModelPermissionsManager()



from __future__ import unicode_literals

from django.utils.unittest import TestCase

from .exceptions import PermAppException
from .utils import get_model


class UtilsTest(TestCase):

    def test_get_model(self):
        with self.assertRaises(PermAppException):
            get_model('does.NotExist', raise_exception=True)
        self.assertEqual(None, get_model('does.NotExist', raise_exception=False))
